import cml
import time
import pandas as pd
from cml import knowledge_searcher

from sklearn.cluster import KMeans
from sklearn.cluster import SpectralClustering
from sklearn.feature_selection import VarianceThreshold
from sklearn.ensemble import ExtraTreesClassifier
from sklearn.tree import DecisionTreeClassifier
from sklearn.cluster import AgglomerativeClustering, DBSCAN
from sklearn.svm import SVC


def generate_blocks():
    df = pd.read_csv("/home/dmt/.cml/data/c_toyset.csv")
    for start in range(0, df.shape[0], 500):
        yield df.iloc[start:start+300]


def demo():

    kmeans = KMeans(precompute_distances=False, algorithm="elkan", n_jobs=2)
    spectral = SpectralClustering(affinity='rbf', n_jobs=2)
    algorithms = [("Dbs", DBSCAN()),
                  ("Agg", AgglomerativeClustering()),
                  ("Kme", kmeans),
                  ("Spe", spectral)]
    constructor = cml.construction("conceptual", algorithms)
    constructor.min_category_size = 1

    variance = VarianceThreshold(2000)
    forest = ExtraTreesClassifier(n_estimators=250, random_state=1)
    selector = cml.feature_selection(filter_method=variance,
                                     embedded_method=forest)

    svc = SVC(gamma="auto", C=1)
    tree = DecisionTreeClassifier(max_depth=5)
    algos = [("Svc", svc), ("Tre", tree)]
    reconstructor = cml.reconstruction("conceptual", algos)
    deconstructor = cml.deconstruction("conceptual")

    start = time.time()
    with knowledge_searcher(constructor,
                            selector,
                            reconstructor,
                            deconstructor,
                            stdout=True,
                            parallel=True) as searcher:
        dbs = []
        halde = []
        for block in generate_blocks():
            knowledge, hld = searcher.search(block)
            dbs.append(knowledge)
            halde.append(hld)

        for hld in halde:
            knowledge, hld = searcher.search(hld)
            dbs.append(knowledge)

    print(dbs)
    end = time.time()
    print(end-start)

    # knowledge = load_knowledge(path)
    # knowledge.dump(path)


if __name__ == '__main__':
    demo()
